%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%ffPIDalgorithm.m
%
%Feedforward+PD algorithm
%Yngvild Aurlien
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function u_ff =ffPID(u)

a_MRU=u(1); %MRU acceleration
v_MRU=u(2); %MRU velocity
x_MRU=u(3); %MRU position
pos_cyl=u(4); %cylinder position
v_cyl=u(5); %cylinder velocity


global e Friction a_ds v_ds x_ds v_c pos_c a_b v_b x_b timestep zero_velocity f_cl F_d F_s M_dyn D k M_CR F_ds F_W a_c i velocity;
global D_CR A_CR A_AA A_A A_p DeltaP M_AA M_d beta Vi1 Vi2 f_valve B_p F_AA F_A F_c F_control a_a v_a pos_a e_a DeltaFP DeltaP_control P1_init P1 P1_next P1_der Q;
global v_f x_f K_v e_v K_f F_b F_heave e_i e_p e_d F_pipes;
global P_N f_valves DeltaP V_i;


F_heave=M_CR*a_MRU; %heaveforce

%calculation of static friction
if abs(F_heave)<F_s
    F_static=F_heave*sign(F_heave);
else
    F_static=F_s*sign(F_heave);
end

%cylinder
if abs(v_c) < 0.05 %static friction
    
    a_c=(F_heave-0.5*F_b-f_cl*v_c-F_pipes-F_static-Friction)/M_CR;
    
else
   
    a_c=(F_heave-0.5*F_b-f_cl*v_c-F_pipes-F_d*sign(v_c)-Friction)/M_CR;
    
end


F_c=a_c*M_CR;


%pipes and valves
F_pipes=(A_p/A_CR)*((DeltaP*A_p+4*f_valve*A_p*A_p^2)*abs(v_c*A_CR/A_p)*(v_c*A_CR/A_p));

%calculation of friction

if abs(v_c) < 0.05 %static friction
    Friction=-F_static-f_cl*v_c-F_pipes;
    
    F_b=2*(F_c+0.5*F_b-F_heave)-k*x_b-D*v_b;

else
    Friction=-F_d*sign(v_c)-f_cl*v_c-F_pipes; %dry friction
    F_b=-k*x_b-D*v_b;

end

%estimation of velocity and position 
v_c=v_c+a_c*timestep;
pos_c=pos_c+v_c*timestep;



%drill string
a_b=F_b/M_dyn;

v_b=v_b+a_b*timestep;
x_b=x_b+v_b*timestep;




%PID
e=pos_cyl-x_MRU;
e_d=v_cyl-v_MRU;
e_i=e_i+e*timestep;

%calculation of control force
F_PID=23500*e_d;
F_control=-Friction*A_CR/A_A-F_PID*A_CR/A_A;



u_ff=zeros(1,1);

u_ff(1)=F_control;
u_ff(2)=pos_c;
u_ff(3)=F_pipes;
u_ff(4)=e_d;
u_ff(5)=e;